// -----------------------------------------------------------------------
// MatchPoint - DKFZ translational registration framework
//
// Copyright (c) German Cancer Research Center (DKFZ),
// Software development for Integrated Diagnostics and Therapy (SIDT).
// ALL RIGHTS RESERVED.
// See mapCopyright.txt or
// http://www.dkfz.de/en/sidt/projects/MatchPoint/copyright.html
//
// This software is distributed WITHOUT ANY WARRANTY; without even
// the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
// PURPOSE.  See the above copyright notices for more information.
//
//------------------------------------------------------------------------
/*!
// @file
// @version $Revision: 797 $ (last changed revision)
// @date    $Date: 2014-10-10 11:42:05 +0200 (Fr, 10 Okt 2014) $ (last change date)
// @author  $Author: floca $ (last changed by)
// Subversion HeadURL: $HeadURL: https://svn.inet.dkfz-heidelberg.de/sbr/Sources/SBR-Projects/MatchPoint/trunk/Code/Core/include/mapMetaPropertyAccessInterface.h $
*/



#ifndef __MAP_META_PROPERTY_ACCESS_INTERFACE_H
#define __MAP_META_PROPERTY_ACCESS_INTERFACE_H

#include "mapMacros.h"

namespace map
{
	namespace core
	{
		/*! @class MetaPropertyLightPointerAccessInterface
		* @brief Interface used to access the value of a MetaProperty as itk::LightObject pointer.
		*
		* This interface should only be implemented for MetaProperties that base on a
		* pointer value type. It allows access to the value in an untyped manner.
		* It is usefull for the situation that you want a value as a down casted type
		* but you don't know its real type. It is used by unwrapMetaProperty_cast().
		*
		* @ingroup Core
		*/
		class MetaPropertyLightPointerAccessInterface
		{
		public:
			/** Smart pointer typedef support. */
			typedef const itk::LightObject*  ConstLightPointer;
			typedef itk::LightObject*        LightPointer;

			/**
			* @brief Returns the property Value as untyped pointer
			* @return Property value.
			* @eguarantee strong
			*/
			virtual bool getUntypedValue(ConstLightPointer&) const = 0;
			/**
			* @brief Returns the property Value as untyped pointer
			* @return Property value.
			* @eguarantee strong
			*/
			virtual bool getUntypedValue(LightPointer&) const = 0;

		protected:

			virtual ~MetaPropertyLightPointerAccessInterface()
			{};

			MetaPropertyLightPointerAccessInterface()
			{};

		private:
			MetaPropertyLightPointerAccessInterface(const
													MetaPropertyLightPointerAccessInterface&);  //purposely not implemented
			void operator=(const MetaPropertyLightPointerAccessInterface&);  //purposely not implemented
		};


		/*! @class MetaPropertyAccessInterface
		* @brief Interface used to access the value of a MetaProperty.
		*
		* This template version is only uzsed if the ValueType of the MetaProperty is no
		* pointer type. In the case of a pointer type the template specialication
		* will be used. This differentiation is needed because the access interface
		* of non pointer types and pointer types must behave different (e.g. the
		* ValueSetType and ValueGetType are different and the concrete implementation
		* of the methods will differ).
		*
		* @ingroup Core
		* @template TValueType Type of the value of the property
		*/
		template <typename TValueType>
		class MetaPropertyAccessInterface
		{
		public:
			/** Smart pointer typedef support. */
			typedef TValueType        ValueType;
			typedef const ValueType&  ValueSetType;
			typedef const ValueType&  ValueGetType;

			/**
			* @brief Returns the property value
			* @return Property value.
			* @eguarantee strong
			*/
			virtual ValueGetType getValue() const = 0;

			/**
			* @brief Sets the property to the passed value
			* @param newValue The new value of the property.
			* @eguarantee strong
			*/
			virtual void setValue(ValueSetType newValue) = 0;

		protected:

			virtual ~MetaPropertyAccessInterface()
			{};

			MetaPropertyAccessInterface()
			{};

		private:
			MetaPropertyAccessInterface(const MetaPropertyAccessInterface<TValueType>
										&); //purposely not implemented
			void operator=(const MetaPropertyAccessInterface<TValueType>&);  //purposely not implemented
		};

		/* Template specialication of the MetaPropertyAccessInterface for pointer types
		*/
		template <typename TValueType>
		class MetaPropertyAccessInterface<TValueType*>
		{
		public:
			/** Smart pointer typedef support. */
			typedef TValueType*        ValueType;
			typedef ValueType          ValueSetType;
			typedef ValueType          ValueGetType;

			/**
			* @brief Returns the property value
			* @return Property value.
			* @eguarantee strong
			*/
			virtual ValueGetType getValue() const = 0;

			/**
			* @brief Sets the property to the passed value
			* @param value The new value of the property.
			* @eguarantee strong
			*/
			virtual void setValue(ValueSetType value) = 0;

		protected:

			virtual ~MetaPropertyAccessInterface()
			{};

			MetaPropertyAccessInterface()
			{};

		private:
			MetaPropertyAccessInterface(const MetaPropertyAccessInterface<TValueType*>
										&);   //purposely not implemented
			void operator=(const MetaPropertyAccessInterface<TValueType*>&);   //purposely not implemented
		};


	}
}

#endif
