// -----------------------------------------------------------------------
// MatchPoint - DKFZ translational registration framework
//
// Copyright (c) German Cancer Research Center (DKFZ),
// Software development for Integrated Diagnostics and Therapy (SIDT).
// ALL RIGHTS RESERVED.
// See mapCopyright.txt or
// http://www.dkfz.de/en/sidt/projects/MatchPoint/copyright.html
//
// This software is distributed WITHOUT ANY WARRANTY; without even
// the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
// PURPOSE.  See the above copyright notices for more information.
//
//------------------------------------------------------------------------
/*!
// @file
// @version $Revision: 797 $ (last changed revision)
// @date    $Date: 2014-10-10 11:42:05 +0200 (Fr, 10 Okt 2014) $ (last change date)
// @author  $Author: floca $ (last changed by)
// Subversion HeadURL: $HeadURL: https://svn.inet.dkfz-heidelberg.de/sbr/Sources/SBR-Projects/MatchPoint/trunk/Code/Core/include/mapMetaPropertyBase.h $
*/



#ifndef __MAP_META_PROPERTY_BASE_H
#define __MAP_META_PROPERTY_BASE_H

#include <typeinfo>
#include <iostream>

#include "itkLightObject.h"

#include "mapMacros.h"
#include "mapMAPCoreExports.h"

namespace map
{
	namespace core
	{
		/*! @class MetaPropertyBase
		* @brief Base class designed as common interface for MetaProperties.
		*
		* MetaProperties are inter alia used to realize a generic streaming of algorithm
		* properties like parameter settings.
		*
		* @ingroup Core
		*/

		class MAPCore_EXPORT MetaPropertyBase : public ::itk::LightObject
		{
		public:
			/** Smart pointer typedef support. */
			typedef MetaPropertyBase  Self;
			typedef ::itk::LightObject  Superclass;
			typedef ::itk::SmartPointer<Self>  Pointer;
			typedef ::itk::SmartPointer<const Self>  ConstPointer;


			/** Run-time type information (and related methods). */
			itkTypeMacro(MetaPropertyBase, LightObject);
			itkCloneMacro(Self);

			/**
			 * @brief Returns the unique type name of the value type that stores the property.
			 * @return A pointer to a const char array containing the unique type name.
			 * @eguarantee strong
			 * @ex
			 */
			virtual const char* getMetaPropertyTypeName() const = 0;

			/**
			 * @brief Returns the type_id of the value type that stores the property.
			 * @eguarantee strong
			 * @return A constant reference to a std::type_info object
			 */
			virtual const std::type_info& getMetaPropertyTypeInfo(void) const = 0;

		protected:
			virtual ~MetaPropertyBase();
			MetaPropertyBase();

			/**
			 * Defines the default behavior for printing out this element
			 * @eguarantee strong
			 * @param os An output stream
			 */
			virtual void PrintSelf(std::ostream& os, itk::Indent indent) const;

		private:
			MetaPropertyBase(const Self&); //purposely not implemented
			void operator=(const Self&);  //purposely not implemented
		};
	}
}

#endif
